#version 330 core
layout (location = 0) in vec3 position;
layout (location = 1) in vec3 normal;
layout (location = 2) in vec2 texCoords;

out vec3 Normal;
out vec3 FragPos;
out vec2 TexCoords;
out vec4 DirShadowCoord;
out vec4 SpotShadowCoord;
out vec4 ProjShadowCoord;
out float Shadowmap_bool;
out float Projector_bool;

uniform mat4 mvp;
uniform mat4 model;
uniform mat4 normalMatrix;


//booleans for toggeling
uniform float spotlight_bool;
uniform float projector_bool;

//shadowmaps
uniform mat4 spot_sm_tex_coord;
uniform mat4 dir_sm_tex_coord;

//projector
uniform mat4 proj_sm_tex_coord;

void main()
{
    gl_Position = mvp * vec4(position, 1.0f);
    FragPos = vec3(model * vec4(position, 1.0f));
    Normal = mat3(normalMatrix) * normal;  
    TexCoords = texCoords;

	DirShadowCoord  = dir_sm_tex_coord  * model * vec4(position , 1.0f);
	
	// Same, but with the light's view matrix
	if(spotlight_bool > 0.5){
		SpotShadowCoord = spot_sm_tex_coord * model * vec4(position , 1.0f);
	}

	if(projector_bool > 0.5){
		ProjShadowCoord = proj_sm_tex_coord * model * vec4(position , 1.0f);
	}
} 