#version 330 core
layout (location = 0) in vec3 position;
layout (location = 1) in vec3 normal;
layout (location = 2) in vec2 texCoords;

out vec3 Normal;
out vec3 FragPos;
out vec2 TexCoords;
out vec4 DirShadowCoord;
out vec4 SpotShadowCoord;
out vec4 ProjShadowCoord;

uniform mat4 mvp;
uniform mat4 model;
//uniform mat4 view;
//uniform mat4 projection;
uniform mat4 normalMatrix;

//shadowmaps
uniform mat4 spot_sm_tex_coord;
uniform mat4 dir_sm_tex_coord;

//projector
uniform mat4 proj_sm_tex_coord;

void main()
{
    gl_Position = mvp * vec4(position, 1.0f);
    FragPos = vec3(model * vec4(position, 1.0f));
    Normal = mat3(normalMatrix) * normal;  
    TexCoords = texCoords;

	// Same, but with the light's view matrix
	DirShadowCoord  = dir_sm_tex_coord  * model * vec4(position , 1.0f);
	SpotShadowCoord = spot_sm_tex_coord * model * vec4(position , 1.0f);
	ProjShadowCoord = proj_sm_tex_coord * model * vec4(position , 1.0f);
} 