#version 330

layout(points) in;
layout(points) out;
layout(max_vertices = 40) out;

in vec3 positionPass[];
in vec3 velocityPass[];
in vec3 colorPass[];
in float lifeTimePass[];
in float sizePass[];
in int typePass[];

out vec3 positionOut;
out vec3 velocityOut;
out vec3 colorOut;
out float lifeTimeOut;
out float sizeOut;
out int typeOut;

uniform vec3 spawnPosition;
uniform vec3 gravity;
uniform vec3 spawnVelocityMin;
uniform vec3 spawnVelocityRange;

uniform vec3 spawnColor;
uniform float spawnSizeMin, spawnSizeRange; 

uniform float lifeTimeMin, lifeTimeRange;
uniform float deltaTime;

uniform vec3 randomSeed;
vec3 localSeed;

uniform int numToSpawn;
uniform int spawnParticles;

float randZeroOne()
{
    uint n = floatBitsToUint(localSeed.y * 214013.0 + localSeed.x * 2531011.0 + localSeed.z * 141251.0);
    n = n * (n * n * 15731u + 789221u);
    n = (n >> 9u) | 0x3F800000u;
 
    float fRes =  2.0 - uintBitsToFloat(n);
    localSeed = vec3(localSeed.x + 147158.0 * fRes, localSeed.y * fRes  + 415161.0 * fRes, localSeed.z + 324154.0 * fRes);
    return fRes;
}

void main()
{
  localSeed = randomSeed;
  // gl_Position doesn't matter now, as rendering is discarded, so don't set it at all

  positionOut = positionPass[0];
  velocityOut = velocityPass[0];
  if(typePass[0] != 0)
  {
	positionOut += velocityOut * deltaTime;
	velocityOut += gravity * deltaTime;
  }

  colorOut = colorPass[0];
  lifeTimeOut = lifeTimePass[0] - deltaTime;
  sizeOut = sizePass[0]; // + sizePass[0] * deltaTime;
  typeOut = typePass[0];
    
  if(typeOut == 0)
  {
    EmitVertex();
    EndPrimitive();
   
	if (spawnParticles != 0)
	{
		for(int i = 0; i < numToSpawn; i++)
		{
		  positionOut = spawnPosition;
		  velocityOut = spawnVelocityMin + vec3(spawnVelocityRange.x * randZeroOne(), spawnVelocityRange.y * randZeroOne(), spawnVelocityRange.z * randZeroOne());
		  colorOut = spawnColor;
		  lifeTimeOut = lifeTimeMin + lifeTimeRange * randZeroOne();
		  sizeOut = spawnSizeMin + spawnSizeRange * randZeroOne();
		  typeOut = 1;
		  EmitVertex();
		  EndPrimitive();
		}
	}
  }
  else if(lifeTimeOut > 0.0)
  {
      EmitVertex();
      EndPrimitive(); 
  }
}